<?php
namespace Utility\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * An attachment (specialized document storage for newsletter functionality).
 * @ORM\Entity
 * @ORM\Table(name="lib_attachment")
 * @ORM\HasLifecycleCallbacks
 */
class Attachment
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=250)
	 */
	protected $filename;

	/**
	 * @ORM\Column(type="string", length=200, name="mime_type")
	 */
	protected $mimeType;

	/**
	 * @ORM\Column(type="integer")
	 */
	protected $downloads = 0;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;


	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @return array
	 */
	public function toArray(array $expand = array())
	{
		return array(
				'id'        => $this->id,
				'filename'  => $this->filename,
				'mimeType'  => $this->mimeType,
				'downloads' => $this->downloads,
				'created'   => !is_null($this->created)
					? $this->created->format(\Utility\Registry::getConfigParam('DateTimeFormat'))
					: null
		);
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['filename'])
			&& $this->filename  = $data['filename'];
		isset($data['mimeType'])
			&& $this->mimeType  = $data['mimeType'];
		isset($data['downloads'])
			&& $this->downloads  = $data['downloads'];
	}

}
