<?php
namespace Utility\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * A notification template.
 * @ORM\Entity
 * @ORM\Table(name="lib_template", uniqueConstraints={@ORM\UniqueConstraint(name="unique_template", columns={"name"})})
 */
class Template
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = false;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", length=50, unique=true)
	 */
	protected $name;

	/**
	 * @ORM\Column(type="string", length=100)
	 */
	protected $subject;

	/**
	 * @ORM\Column(type="text")
	 */
	protected $tags;

	/**
	 * @ORM\Column(type="text", name="email_template")
	 */
	protected $emailTemplate;

	/**
	 * @ORM\Column(type="text", name="sms_template")
	 */
	protected $smsTemplate;

	/**
	 * @ORM\OneToOne(targetEntity="RepeaterTemplate")
	 * @ORM\JoinColumn(name="lib_repeater_template_id")
	 **/
	protected $repeaterTemplate;


	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @return array
	 */
	public function toArray(array $expand = array('repeaterTemplate'))
	{
		return array(
				'id'               => $this->id,
				'name'             => $this->name,
				'subject'          => $this->subject,
				'tags'             => $this->tags,
				'emailTemplate'    => $this->emailTemplate,
				'smsTemplate'      => $this->smsTemplate,
				'repeaterTemplate' => in_array('repeaterTemplate', $expand)
										&& !is_null($this->repeaterTemplate)
					? $this->repeaterTemplate->toArray($expand)
					: null
		);
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['name'])
			&& $this->name  = $data['name'];
		isset($data['subject'])
			&& $this->subject  = $data['subject'];
		isset($data['tags'])
			&& $this->tags  = $data['tags'];
		isset($data['emailTemplate'])
			&& $this->emailTemplate  = $data['emailTemplate'];
		isset($data['smsTemplate'])
			&& $this->smsTemplate  = $data['smsTemplate'];
		isset($data['repeaterTemplate'])
			&& $this->repeaterTemplate  = $data['repeaterTemplate'];
	}

}
